import tensorflow as tf
from tensorflow.keras.layers import Input
from tensorflow.keras.models import Model, load_model
import numpy as np
import cv2

model_name = 'keras_mnist_autoencoder_model.h5'
autoencoder = load_model(model_name)

# encoder
orig_input = Input(shape=(28,28,1), dtype=float)
x = orig_input
for layer in autoencoder.layers[1:8]:
    x = layer(x)

encoder = Model(orig_input, x)
encoder.summary()

# load one image
image = cv2.imread('inp'+str(0).zfill(5)+".png",cv2.IMREAD_GRAYSCALE)
input_images = (image.astype(np.float32)/255.0).reshape([1,28,28,1])
encoded = encoder.predict(input_images)

# decoder
coded_input = Input(shape=(128,), dtype=float)
x = coded_input
for layer in autoencoder.layers[8:]:
    x = layer(x)

decoder = Model(coded_input, x)
decoder.summary()

# here copy a real code provided by encoder
features = encoded[0]

last_features = np.copy(features)
last_features[0] = 1.0 # modify somehow

cv2.namedWindow("generator")
cv2.imshow('generator',np.zeros((560,560),np.uint8))

# create trackbars
index = 55
feature = 0

def update_index( *args ):
    global index, feature
    index = args[0]
    #features[index] = feature

def update_feature( *args ):
    global index, features
    feature = float(args[0]) / 127.0
    features[index] = feature

cv2.createTrackbar("index", "generator", index, 127, update_index)
cv2.createTrackbar("value", "generator", feature, 127, update_feature)

# play with decoder based generator
while True:

    if not np.array_equal(features,last_features):
        last_features = np.copy(features)
        coded = np.array([features],np.float32)
        decoded = decoder.predict(coded)
        decoded = np.asarray(decoded[0]*255,np.uint8)
        decoded = cv2.resize(decoded,(560,560))
        cv2.imshow('generator',decoded)

    if cv2.waitKey(10) == 27: # press Esc to exit
        break

cv2.destroyAllWindows()
